/*
 *  Copyright (C) 2020. Maximilian Keppeler (https://www.maxkeppeler.com)
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.maxkeppeler.bottomsheets.input.type

import android.os.Bundle
import android.text.InputFilter
import android.text.InputType
import androidx.annotation.StringRes

/** Listener which returns the new text and if it is final. */
typealias EditTextInputListener = (value: String?) -> Unit

@Suppress("unused")
class InputEditText(key: String? = null, func: InputEditText.() -> Unit) : Input(key) {

    init {
        func()
    }

    private var changeListener: EditTextInputListener? = null
    private var resultListener: EditTextInputListener? = null

    internal var hint: String? = null
        private set

    internal var hintRes: Int? = null
        private set

    internal var inputType: Int = InputType.TYPE_CLASS_TEXT
        private set

    internal var inputFilter: InputFilter? = null
        private set

    internal var defaultText: String? = null
        private set

    internal var defaultTextRes: Int? = null
        private set

    var value: String? = null
        internal set(value) {
            changeListener?.invoke(value)
            field = value
        }

    fun default(@StringRes defaultTextRes: Int) {
        this.hintRes = defaultTextRes
    }

    fun default(defaultText: String) {
        this.defaultText = defaultText
    }

    fun hint(@StringRes hintRes: Int) {
        this.hintRes = hintRes
    }

    fun hint(hint: String) {
        this.hint = hint
    }

    fun changeListener(listener: EditTextInputListener) {
        this.changeListener = listener
    }

    fun resultListener(listener: EditTextInputListener) {
        this.resultListener = listener
    }

    fun inputType(inputType: Int) {
        this.inputType = inputType
    }

    fun inputFilter(inputFilter: InputFilter) {
        this.inputFilter = inputFilter
    }

    override fun valid(): Boolean = !defaultText.isNullOrEmpty()
            || !value.isNullOrEmpty()

    override fun invokeResultListener() =
        resultListener?.invoke(value)

    override fun putValue(bundle: Bundle, index: Int) {
        bundle.putString(getKeyOrIndex(index), value)
    }
}